import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:supper_app/src/services/storage_service/storage_service.dart';
import 'package:get_it/get_it.dart';

enum AppTheme {
  light,
  dark,
}

enum AppLanguage { en, vi }

class AppState<T extends AppTheme, V extends AppLanguage> extends Equatable {
  final T appTheme;
  final V? appLanguage;

  const AppState({required this.appTheme, this.appLanguage});

  @override
  List<Object?> get props => [appTheme, appLanguage];
}

class AppCubit extends Cubit<AppState> {
  AppCubit(AppState<AppTheme, AppLanguage> appState)
      : super(const AppState(
            appTheme: AppTheme.light, appLanguage: AppLanguage.en));

  Future<bool> saveLanguage(String langCode) {
    final storageService = GetIt.instance<StorageService>();
    return storageService.shared.setString('language', langCode);
  }

  Future<bool> saveTheme(String langCode) {
    final storageService = GetIt.instance<StorageService>();
    return storageService.shared.setString('theme', langCode);
  }

  void changeTheme({required AppTheme theme}) {
    saveTheme(theme.toString());
    emit(AppState(appTheme: theme, appLanguage: state.appLanguage));
  }

  void changeLanguage({required AppLanguage language}) {
    saveLanguage(language.toString());
    emit(AppState(appLanguage: language, appTheme: state.appTheme));
  }
}
